var appDashboard = angular.module('appDashboard', [
    'ngRoute',
    'enplug.sdk',
    'enplug.utils',
    'ngDialog'
]);

appDashboard.config(function ($routeProvider, $locationProvider, $httpProvider){
    'use strict';
    $locationProvider.html5Mode(false);

    $routeProvider.when('/', {
        templateUrl: 'templates/list.html',
        controller: 'ListAssetsController'
    }).when('/create', {
        templateUrl: 'templates/form.html',
        controller: 'StoreAssetsController'
    }).when('/edit/:assetId', {
        templateUrl: 'templates/form.html',
        controller: 'UpdateAssetsController'
    });
});

appDashboard.factory('AssetService', function ($enplugAccount, $enplugDashboard, $route) {
    'use strict';
    var assetService = {
        validate: function (asset) {
            if (asset.Value.Name == '') {
                return 'Nome da página é obrigatório.';
            }

            if (asset.Value.HtmlContent == '') {
                return 'O HTML/Javascript é obrigatório.';
            }

            return '';
        },
        persistAsset: function (asset) {
            var errorMsg = assetService.validate(asset),
                options = {
                    initialTab: 'displays',
                    showSchedule: true,
                    scheduleOptions: {
                        showDuration: true
                    },
                    loadingMessage: 'Salvando...',
                    successMessage: 'Nova página salva com sucesso!',
                    errorMessage: 'O nova página não foi salva, tente novamente.',
                    showDeployDialog: true//Default é true...
                };

            if (errorMsg != '') {
                $enplugDashboard.errorIndicator(errorMsg);
                return;
            }

            if (asset.Id != null) {
                options.loadingMessage = 'Atualizando...';
                options.successMessage = 'Página atualizada com sucesso!';
                options.errorMessage = 'A página não foi atualizada, tente novamente.';
                options.showDeployDialog = false;//... porém no "update" impedirá o processo se já estiver setado e nada for alterado...
            }

            if (asset.VenueIds.length <= 0) {
                options.showDeployDialog = true;//...no entanto se for zero pode ser exibido mesmo no "update"!
            }

            return $enplugAccount.saveAsset(asset, options);
        },
        deleteAsset: function (asset) {
            $enplugDashboard.openConfirm({
                title: 'Deletar página',
                text: 'Tem certeza de que deseja excluir a página <strong>"' + asset.Value.Name + '"</strong>?',
                cancelText: 'Cancelar',
                confirmText: 'Deletar',
                confirmClass: 'btn-danger'
            }).then(function () {
                $enplugAccount.deleteAsset(asset.Id).then(function (response) {
                    $enplugDashboard.successIndicator('Página "' + asset.Value.Name + '" excluída com sucesso!');
                    $route.reload();
                }, function () {
                    $enplugDashboard.errorIndicator('Não foi possível excluir a página, tente novamente.');
                });
            });
        },
        deployAsset: function (asset) {
            var options = {
                initialTab: 'displays',
                showSchedule: true,
                scheduleOptions: {
                    showDuration: true
                },
                loadingMessage: 'Aguarde...',
                successMessage: 'Instalação da aplicação "' + asset.Value.Name + '" concluída com sucesso!',
                errorMessage: 'Falha na instalação da aplicação, tente novamente.',
                showDeployDialog: true//No deploy SEMPRE deve ser exibido o modal com as telas disponíveis!!!
            };

            $enplugAccount.saveAsset(asset, options).then(function (response) {
                $route.reload();
            });
        }
    };

    return assetService;
});

appDashboard.controller('ListAssetsController', function($scope, $location, $enplugDashboard, $enplugAccount, AssetService){
    'use strict';
    $scope.assets = [];// usado na "list" view!
    $enplugDashboard.setHeaderTitle('');

    $enplugAccount.getAssets().then(function (_assets) {
        $scope.assets = _assets;

        if ($scope.assets.length > 0) {
            $enplugDashboard.setHeaderButtons([
                {text:' Criar página', action:$scope.createAsset, class:'btn btn-primary ion-android-add-circle'}
            ]);
        } else {
            $enplugDashboard.setHeaderButtons([]);
        }

        $enplugDashboard.pageLoading(false);
    }, function () {
        $enplugDashboard.errorIndicator('Falha no carregamento das páginas, tente novamente.');
    });

    // helper methods
    $scope.createAsset = function () {
        $enplugDashboard.pageLoading(true);
        $location.path('/create');
    };

    $scope.editAsset = function (id) {
        $enplugDashboard.pageLoading(true);
        $location.path('/edit/' + id);
    };

    $scope.deployAsset = function (asset) {
        AssetService.deployAsset(asset);
    };

    $scope.deleteAsset = function (asset) {
        AssetService.deleteAsset(asset);
    };
});

appDashboard.controller('StoreAssetsController', function($scope, $location, $enplugDashboard, $enplugAccount, AssetService, ngDialog, $http, $window, $rootScope){
    'use strict';
    $scope.asset = {
        Id: null,
        VenueIds: [],
        Duration:30,
        Value: {
            Name: '',
            HtmlContent:''
        }
    };

    /**
     * "init view"
     *
     * DICA: Se colocar "()" na "action" do "setHeaderButtons", o navegador irá executar o método imediatamente após a página carregar,
     *   por este motivo que "Minhas páginas" passa o método do service direto na action e "Salvar", que precisa enviar o "asset" para o
     *   service, não passa o método do service direto e utiliza o alias "saveAsset" que não recebe parametro nenhum o/
     *
     * DICA 2: Não tem como passar o $scope para o service, ou melhor, é complicado porque não é recomendado =]
     */
    $enplugDashboard.setHeaderTitle('Criar página');
    $enplugDashboard.setHeaderButtons([
        {text:' Minhas páginas', action:listAssets, class: 'btn btn-default ion-android-list'},
        {text:' Salvar', action:saveAsset, class: 'btn btn-primary ion-checkmark'}
    ]);
    $enplugDashboard.pageLoading(false);

    // helper methods
    function listAssets(){
        ngDialog.closeAll();
        $location.path('/');
        return;
    };

    function saveAsset(){
        ngDialog.closeAll();
        
		AssetService.persistAsset($scope.asset).then(function(response){
            $location.path('/');
        });
    };
});

appDashboard.controller('UpdateAssetsController', function ($scope, $location, $route, $routeParams, $enplugDashboard, $enplugAccount, AssetService, ngDialog, $http, $window, $rootScope){
    'use strict';
    if (typeof $routeParams.assetId == 'undefined') {
        $enplugDashboard.errorIndicator('A página solicitada não existe, tente novamente.');
        $location.path('/');
        return;
    }

    $scope.asset = null;

    $enplugAccount.getAssets().then(function (_assets) {
        if (_assets.length > 0) {
            angular.forEach(_assets, function (item) {
                if (item.Id == $routeParams.assetId) {
                    $scope.asset = item;
                    return false;
                }
            });
        }

        if (!$scope.asset) {
            $enplugDashboard.errorIndicator('A página solicitada não está mais disponível, tente novamente.');
            $location.path('/');
            return;
        }

        // "init view"
        $enplugDashboard.setHeaderTitle('Editar página "' + $scope.asset.Value.Name + '"');
        $enplugDashboard.setHeaderButtons([
            {text:' Minhas páginas', action:listAssets, class: 'btn btn-default ion-android-list'},
            {text:' Criar página',action:createAsset, class:'btn btn-default ion-android-add-circle'},
            {text:' Atualizar', action:updateAsset, class: 'btn btn-primary ion-checkmark'}
        ]);

        $enplugDashboard.pageLoading(false);
    }, function () {
        $enplugDashboard.errorIndicator('Não foi possível carregar a página selecionada, tente novamente.');
        $location.path('/');
    });

    // helper methods
    function listAssets(){
        ngDialog.closeAll();
        $location.path('/');
        return;
    };

    function createAsset(){
        ngDialog.closeAll();
        $enplugDashboard.pageLoading(true);
        $location.path('/create');
    };
    
    function updateAsset(){
        ngDialog.closeAll();

        AssetService.persistAsset($scope.asset).then(function (response) {
            $route.reload();
        });
    };
});